#include <stdio.h>
#include </usr/include/stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/time.h>


int main (int argc, char** argv)
/*
 * Compare two files byte-for-byte to see if they are the same.
 *  file1    -name of first file to read
 *  file2    -name of second file to read
 */
{
#define CMP_BUF_SIZE 524288
   int i;           /* loop index */
   int in1Fd;       /* open file descriptors */
   int in2Fd;
   char* buf1;      /* read buffers */
   char* buf2;
   int n1,n2;       /* I/O count */
   int minn;        /* minimum of n1 and n2 */
   long int bpos;   /* byte position of error */
   char* file1;
   char* file2;
   struct timeval tp1;
   struct timeval tp2;
   float time;      /* time difference in seconds */
   float rate;      /* rate at which we are reading from file1 */
   long totbytes;
   float tottime;
   long cumbytes;
   float cumtime;


   if (argc<2)  {
      fprintf(stderr,"Usage:  cmpmeter file1 file2\n");
      exit(1);
   }

   file1=argv[1];
   file2=argv[2];

   in1Fd = open (file1, O_RDONLY, 0);
   if (in1Fd < 0)  {
      fprintf (stderr,"Can't open \"%s\": ", file1);
      perror("");
      return (-1);
   }
   in2Fd = open (file2, O_RDONLY, 0);
   if (in2Fd < 0)  {
      close (in1Fd);
      fprintf (stderr,"Can't open \"%s\": ", file2);
      perror("");
      return (-1);
   }

   buf1=malloc(CMP_BUF_SIZE);
   buf2=malloc(CMP_BUF_SIZE);
   if (buf1==NULL || buf2==NULL)  {
      fprintf (stderr,"Could not malloc buffers!\n");
      close(in1Fd);
      close(in2Fd);
      return(-1);
   }

   totbytes=0;
   cumbytes=0;
   tottime=0;
   cumtime=0;
   bpos=1;   /* number characters starting at 1, not 0 */

   gettimeofday(&tp1,NULL);
   n1=read(in1Fd, buf1, CMP_BUF_SIZE);
   gettimeofday(&tp2,NULL);

   n2=read(in2Fd, buf2, CMP_BUF_SIZE);

   while (n1>0 && n2>0)  {
      /* compute rate info in KB/s */
     
      time=(tp2.tv_sec - tp1.tv_sec)
              + (float)(tp2.tv_usec - tp1.tv_usec)/1e6;
      tottime+=time;
      cumtime+=time;
      totbytes+=n1;
      cumbytes+=n1;
      if (tottime>=1)  {
         rate=totbytes/tottime/1024.;
         printf("%g      \r",rate);
         fflush(stdout);
         totbytes=0;
         tottime=0;
      }

      minn= (n1<n2 ? n1 : n2);
      if (bcmp(buf1,buf2,minn)!=0)  {
         for (i=0; i<minn; i++)  {
            if (buf1[i]!=buf2[i])  {
               bpos+=i;
               break;    /* exit for i loop */
            }
         }
         printf("\n%s and %s differ at char %d\n",file1,file2,bpos);
         break;   /* exit while loop */
      }
      
      bpos+=minn;

      if (n1!=n2)  {
         /* one of the following will be 0 */
         n1-=minn;
         n2-=minn;
      }
      else  {
         gettimeofday(&tp1,NULL);
         n1=read(in1Fd, buf1, CMP_BUF_SIZE);
         gettimeofday(&tp2,NULL);

         n2=read(in2Fd, buf2, CMP_BUF_SIZE);
      }
   }

   if (n1<0)  {
      fprintf (stderr,"\nI/O Error reading %s at char %d\n",file1,bpos);
   }
   else if (n1==0 && n2>0)  {
      fprintf (stderr,"\nEOF on %s at char %d\n",file1,bpos);
   }

   if (n2<0)  {
      fprintf (stderr,"\nI/O Error reading %s at char %d\n",file2,bpos);
   }
   else if (n2==0 && n1>0)  {
      fprintf (stderr,"\nEOF on %s at char %d\n",file2,bpos);
   }

   if (cumtime>0)  {
      rate=cumbytes/cumtime/1024.;
      printf("\nAverage file1 read rate: %g KBytes/sec\n",rate);
   }

   close(in1Fd);
   close(in2Fd);
   free(buf1);
   free(buf2);
   return (0);
}
