#include <stdio.h>
#include </usr/include/stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/time.h>


int main (int argc, char** argv)
/*
 * Copy two files and compute the rate of copying.
 *  file1    -name of first file to read
 *  file2    -name of second file to read
 */
{
#define CP_BUF_SIZE 524288
   int inFd;       /* open file descriptors */
   int outFd;
   char* buf;      /* read buffer */
   int n1,n2;       /* I/O count */
   char* file1;
   char* file2;
   struct timeval tp1;
   struct timeval tp2;
   float time;      /* time difference in seconds */
   float rate;      /* rate at which we are reading from file1 */
   long totbytes;
   float tottime;
   long cumbytes;
   float cumtime;
   int status = 0;
   int meastwo;


   if (argc<2)  {
      fprintf(stderr,"Usage:  cpmeter file1 file2 [1|2]\n");
      exit(1);
   }

   file1=argv[1];
   file2=argv[2];

   meastwo = (argv[3]!=NULL && argv[3][0]=='2' && argv[3][1]==NULL);

   inFd = open (file1, O_RDONLY, 0);
   if (inFd < 0)  {
      fprintf (stderr,"Can't open \"%s\": ", file1);
      perror("");
      return (-1);
   }
   outFd = open (file2, O_WRONLY|O_CREAT, 0666);
   if (outFd < 0)  {
      close (inFd);
      fprintf (stderr,"Can't open \"%s\": ", file2);
      perror("");
      return (-1);
   }

   buf=malloc(CP_BUF_SIZE);
   if (buf==NULL)  {
      fprintf (stderr,"Could not malloc buffer!\n");
      close(inFd);
      close(outFd);
      return(-1);
   }

   totbytes=0;
   cumbytes=0;
   tottime=0;
   cumtime=0;

   while (1)  {

      if (!meastwo)
         gettimeofday(&tp1,NULL);
      n1=read(inFd, buf, CP_BUF_SIZE);
      if (!meastwo)
         gettimeofday(&tp2,NULL);
      if (n1<=0)  {
         if (n1<0)  {
            fprintf (stderr,"\nI/O Error reading %s: ",file1);
            perror("");
            status=-1;
         }
         break;
      }

      if (meastwo)
         gettimeofday(&tp1,NULL);
      n2=write(outFd, buf, n1);
      if (meastwo)
         gettimeofday(&tp2,NULL);
      if (n2!=n1)  {
         if (n2<0)  {
            fprintf (stderr,"\nI/O Error writing %s: ",file2);
            perror("");
            status=-1;
         }
         break;
      }

      /* compute rate info in KB/s */
     
      time=(tp2.tv_sec - tp1.tv_sec)
              + (float)(tp2.tv_usec - tp1.tv_usec)/1e6;
      tottime+=time;
      cumtime+=time;
      totbytes+=n1;
      cumbytes+=n1;
      if (tottime>=1)  {
         rate=totbytes/tottime/1024.;
         printf("%g      \r",rate);
         fflush(stdout);
         totbytes=0;
         tottime=0;
      }
   }

   if (cumtime>0)  {
      rate=cumbytes/cumtime/1024.;
      printf("\nAverage %s rate: %g KBytes/sec\n",
                                      (meastwo ? "write" : "read"), rate);
   }

   close(inFd);
   close(outFd);
   free(buf);
   return(status);
}
